<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}

/**
 * Iconic_WW_Endpoints.
 *
 * @class    Iconic_WW_Endpoints
 * @version  1.0.0
 * @category Class
 * @author   Iconic
 */
class Iconic_WW_Endpoints {
	/**
	 * Current endpoint.
	 *
	 * @var bool|string
	 */
	public static $current_endpoint = false;

	/**
	 * Run.
	 */
	public static function run() {
		// Global.
		add_filter( 'init', array( __CLASS__, 'add_endpoints' ) );
		add_action( 'init', array( __CLASS__, 'add_rewrite_rules' ) );

		// Frontend.
		if ( ! is_admin() ) {
			add_action( 'template_redirect', array( __CLASS__, 'set_endpoint' ), 1 );
			add_action( 'template_redirect', array( __CLASS__, 'redirect' ) );
			add_filter( 'request', array( __CLASS__, 'filter_request' ) );
			add_filter( 'woocommerce_get_breadcrumb', array( __CLASS__, 'get_breadcrumb' ), 10, 2 );
			add_filter( 'the_title', array( __CLASS__, 'the_title' ), 10, 2 );
			add_filter( 'document_title_parts', array( __CLASS__, 'page_title' ) );
			add_filter( 'is_woocommerce', array( __CLASS__, 'is_woocommerce' ) );
			self::content();
		}
	}

	/**
	 * Is this one of our endpoints.
	 *
	 * @return bool
	 */
	public static function is_endpoint() {
		$current_uri = get_page_uri();
		$base        = Iconic_WW_Wishlists::get_wishlists_base_slug();

		if ( ! $base ) {
			return false;
		}

		return strpos( $current_uri, $base ) === 0;
	}

	/**
	 * Set endpoint var.
	 */
	public static function set_endpoint() {
		if ( ! self::is_endpoint() ) {
			return;
		}

		if ( is_singular( 'iconic_ww_wishlist' ) ) {
			self::$current_endpoint = 'view';

			return;
		}

		$endpoints = self::get_endpoints();

		if ( ! $endpoints ) {
			return;
		}

		foreach ( $endpoints as $endpoint_key => $endpoint ) {
			$query_var = get_query_var( $endpoint, null );

			if ( is_null( $query_var ) ) {
				continue;
			}

			self::$current_endpoint = $endpoint_key;

			return;
		}

		self::$current_endpoint = 'base';
	}

	/**
	 * Get current endpoint info.
	 *
	 * @return array|bool
	 */
	public static function get_current_endpoint_data() {
		$endpoints = self::get_endpoints();

		if ( ! $endpoints || ! isset( $endpoints[ self::$current_endpoint ] ) ) {
			return false;
		}

		return array(
			'key'  => self::$current_endpoint,
			'path' => $endpoints[ self::$current_endpoint ],
		);
	}

	/**
	 * Redirect endpoints when required.
	 */
	public static function redirect() {
		if ( ! self::$current_endpoint ) {
			return;
		}

		// Redirect to "find" if view is empty.
		if ( self::is( 'view' ) && empty( get_query_var( 'view', null ) ) ) {
			wp_safe_redirect( self::get_endpoint_url( 'find' ), 302 );
			exit;
		}

		// Redirect to "find" if search is empty.
		if ( self::is( 'find' ) && ( isset( $_GET['search'] ) && empty( $_GET['search'] ) ) ) {
			wp_safe_redirect( self::get_endpoint_url( 'find' ), 302 );
			exit;
		}

		if ( is_user_logged_in() ) {
			return;
		}

		// Redirect to "find" if not logged in.
		if ( self::is( 'base' ) ) {
			wp_safe_redirect( self::get_endpoint_url( 'find' ), 302 );
			exit;
		}
	}

	/**
	 * Get all endpoints.
	 *
	 * @return array|bool
	 */
	public static function get_endpoints() {
		$base = Iconic_WW_Wishlists::get_wishlists_base_slug();

		if ( ! $base ) {
			return false;
		}

		return apply_filters( 'iconic_ww_endpoints', array(
			'view' => $base . '/view',
			'find' => $base . '/find',
			'base' => $base,
		) );
	}

	/**
	 * Get endpoint URL.
	 *
	 * @param $endpoint
	 *
	 * @return mixed
	 */
	public static function get_endpoint_url( $endpoint ) {
		$endpoints     = self::get_endpoints();

		if ( ! $endpoints ) {
			return '';
		}

		$endpoint_path = isset( $endpoints[ $endpoint ] ) ? $endpoints[ $endpoint ] : false;

		if ( ! $endpoint_path ) {
			return '';
		}

		return trailingslashit( home_url( $endpoint_path ) );
	}

	/**
	 * Add endpoints.
	 */
	public static function add_endpoints() {
		$endpoints = self::get_endpoints();

		if ( ! $endpoints ) {
			return;
		}

		foreach ( $endpoints as $endpoint_key => $endpoint ) {
			if ( $endpoint_key === 'base' ) {
				continue;
			}

			add_rewrite_endpoint( $endpoint_key, EP_PERMALINK | EP_PAGES );
		}
	}

	/**
	 * Add rewrite rules.
	 */
	public static function add_rewrite_rules() {
		$endpoints = self::get_endpoints();

		if ( ! $endpoints ) {
			return;
		}

		add_rewrite_rule( $endpoints['find'] . '/page/([0-9]{1,})/?$', 'index.php?find=&paged=$matches[1]&pagename=' . $endpoints['base'], 'top' );
	}

	/**
	 * Set query var to true if present, so no
	 * sub-page is required.
	 *
	 * @param array $vars
	 *
	 * @return array
	 */
	public static function filter_request( $vars ) {
		$base = Iconic_WW_Wishlists::get_wishlists_base_slug();

		if ( ! $base || ! isset( $vars['pagename'] ) || $vars['pagename'] !== $base ) {
			return $vars;
		}

		$endpoints = self::get_endpoints();

		if ( ! $endpoints ) {
			return $vars;
		}

		foreach ( $endpoints as $endpoint_key => $endpoint ) {
			if ( ! isset( $vars[ $endpoint_key ] ) ) {
				continue;
			}

			if ( $endpoint_key === 'view' ) {
				// If on one of these endpoints and value is empty,
				// set to `false` so we can redirect to base.
				$vars[ $endpoint ] = empty( $vars[ $endpoint_key ] ) ? false : $vars[ $endpoint_key ];

				return $vars;
			}

			if ( $endpoint_key === 'base' ) {
				// If on one of these endpoints and value is empty, set to
				// `true` as it is allowed or allow the sub page to be set.
				$vars[ $endpoint ] = empty( $vars[ $endpoint_key ] ) ? true : $vars[ $endpoint_key ];

				return $vars;
			}

			// Otherwise, set the value to true if it's empty.
			// If it isn't empty, set to false so we can redirect.
			// These endpoints shouldn't have children.
			$vars[ $endpoint ] = empty( $vars[ $endpoint_key ] ) ? true : false;

			return $vars;
		}

		// Not one of our endpoints.
		return $vars;
	}

	/**
	 * Is endpoint?
	 *
	 * @param string $endpoint
	 *
	 * @return bool
	 */
	public static function is( $endpoint ) {
		return $endpoint === self::$current_endpoint;
	}

	/**
	 * Add page to breadcrumbs.
	 *
	 * @param array         $crumbs
	 * @param WC_Breadcrumb $breadcrumbs
	 *
	 * @return array
	 */
	public static function get_breadcrumb( $crumbs, $breadcrumbs ) {
		if ( self::$current_endpoint === 'base' ) {
			return $crumbs;
		}

		foreach ( $crumbs as $key => $crumb ) {
			if ( empty( $crumb[0] ) ) {
				unset( $crumbs[ $key ] );
			}
		}

		$current = count( $crumbs ) > 1 ? array_pop( $crumbs ) : false;

		if ( self::is( 'find' ) ) {
			$paged = get_query_var( 'paged', null );

			if ( $paged > 0 ) {
				array_pop( $crumbs );
				$crumbs['iconic_ww'] = array( self::get_page_title( 'find' ), self::get_endpoint_url( 'find' ) );
			}
		} elseif ( self::is( 'view' ) ) {
			$user     = new Iconic_WW_User();
			$wishlist = new Iconic_WW_Wishlist();

			if ( ! $user->is_author( $wishlist ) ) {
				$crumbs['iconic_ww'] = array( self::get_page_title( 'find' ), self::get_endpoint_url( 'find' ) );
			} else {
				$crumbs['iconic_ww'] = array( self::get_page_title( 'base' ), self::get_endpoint_url( 'base' ) );
			}
		}

		if ( $current ) {
			$crumbs[] = $current;
		}

		return array_values( $crumbs );
	}

	/**
	 * Add endpoint content.
	 */
	public static function content() {
		// Your wishlists.
		add_action( 'iconic_ww_content_base', array( __CLASS__, 'content_your_wishlists' ), 10 );

		// View wishlist.
		add_action( 'iconic_ww_content_view', array( __CLASS__, 'content_wishlist_meta' ), 10 );
		add_action( 'iconic_ww_content_view', array( __CLASS__, 'content_wishlist_items' ), 20 );

		// Find wishlists.
		add_action( 'iconic_ww_content_find', array( __CLASS__, 'content_search_wishlists' ), 10 );
		add_action( 'iconic_ww_content_find', array( __CLASS__, 'content_find_wishlists' ), 20 );
	}

	/**
	 * Display your wishlists.
	 */
	public static function content_your_wishlists() {
		$wishlists = Iconic_WW_Wishlists::get_wishlists();

		Iconic_WW_Helpers::include_template( 'loop/wishlists', array(
			'wishlists'      => $wishlists,
			'user'           => new Iconic_WW_User(),
			'posts_per_page' => - 1,
		) );
	}

	/**
	 * Search wishlists.
	 */
	public static function content_search_wishlists() {
		Iconic_WW_Helpers::include_template( 'components/search-wishlists' );
	}

	/**
	 * Display all wishlists.
	 */
	public static function content_find_wishlists() {
		$wishlists = Iconic_WW_Wishlists::get_wishlists( array(
			'author'      => false,
			'post_status' => 'publish',
		) );

		Iconic_WW_Helpers::include_template( 'loop/wishlists', array(
			'wishlists' => $wishlists,
		) );
	}

	/**
	 * Wishlist author.
	 */
	public static function content_wishlist_meta() {
		$wishlist = new Iconic_WW_Wishlist();

		if ( ! $wishlist ) {
			return;
		}

		$user = new Iconic_WW_User();

		Iconic_WW_Helpers::include_template( 'components/wishlist-meta', array(
			'wishlist' => $wishlist,
			'user'     => $user,
		) );
	}

	/**
	 * Display wishlist items.
	 */
	public static function content_wishlist_items() {
		$wishlist = new Iconic_WW_Wishlist();

		if ( ! $wishlist ) {
			return;
		}

		$items = $wishlist->get( 'items' );
		$user  = new Iconic_WW_User();

		Iconic_WW_Helpers::include_template( 'loop/wishlist-items', array(
			'wishlist' => $wishlist,
			'items'    => $items,
			'user'     => $user,
		) );
	}

	/**
	 * Modify page title.
	 *
	 * @param $title_parts
	 *
	 * @return array
	 */
	public static function page_title( $title_parts ) {
		if ( ! self::$current_endpoint ) {
			return $title_parts;
		}

		$title_parts['title'] = self::get_current_page_title( true );

		return $title_parts;
	}

	/**
	 * Modify page title.
	 *
	 * @param string $title
	 * @param null   $id
	 *
	 * @return bool|string
	 */
	public static function the_title( $title, $id = null ) {
		$base_page_id = Iconic_WW_Wishlists::get_wishlists_base_page_id();

		if ( $id !== $base_page_id ) {
			return $title;
		}

		return self::get_current_page_title();
	}

	/**
	 * Get current page title.
	 *
	 * @param bool $document Is this a document title?
	 *
	 * @return bool|string
	 */
	public static function get_current_page_title( $document = false ) {
		return self::get_page_title( self::$current_endpoint, $document );
	}

	/**
	 * Get page title.
	 *
	 * @param string $endpoint
	 * @param bool   $document Is this a document title?
	 *
	 * @return bool|string
	 */
	public static function get_page_title( $endpoint, $document = false ) {
		$title = false;

		if ( ! $endpoint ) {
			return apply_filters( 'iconic_ww_page_title', $title );
		}

		$labels = Iconic_WW_Wishlists::get_wishlists_label();

		switch ( $endpoint ) {
			case 'view':
				$wishlist = new Iconic_WW_Wishlist();
				$sep      = apply_filters( 'document_title_separator', '-' );
				$title    = $document ? sprintf( '%s %s %s', $wishlist->get( 'post_title' ), $sep, $labels['singular'] ) : $wishlist->get( 'post_title' );
				break;
			case 'find':
				$title = sprintf( '%s %s', __( 'Find', 'iconic-ww' ), $labels['singular'] );
				break;
				break;
			case 'base':
				$title = sprintf( '%s %s', __( 'Your', 'iconic-ww' ), $labels['plural'] );
				break;
			default:
				$title = $labels['plural'];
				break;
		}

		return apply_filters( 'iconic_ww_page_title', $title );
	}

	/**
	 * Make sure wishlist pages are registered as woocommerce pages.
	 *
	 * @param bool $is_woocommerce
	 *
	 * @return bool
	 */
	public static function is_woocommerce( $is_woocommerce ) {
		if ( ! self::$current_endpoint ) {
			return $is_woocommerce;
		}

		return true;
	}
}