<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}

/**
 * Iconic_WSSV_Helpers.
 *
 * @class    Iconic_WSSV_Helpers
 * @version  1.0.0
 * @author   Iconic
 */
class Iconic_WW_Helpers {
	/**
	 * Get template path.
	 *
	 * @param string $path
	 *
	 * @return string
	 */
	public static function get_template_path( $path ) {
		return ICONIC_WW_TPL_PATH . $path . '.php';
	}

	/**
	 * Include template.
	 *
	 * @param string $path
	 * @param array  $params
	 * @param bool   $once
	 */
	public static function include_template( $path, $params = array(), $once = false ) {
		if ( ! empty( $params ) && is_array( $params ) ) {
			extract( $params );
		}

		$path = self::get_template_path( $path );

		if ( ! file_exists( $path ) ) {
			return;
		}

		if ( $once ) {
			include_once( $path );
			return;
		}

		include( $path );
	}

	/**
	 * Get template html.
	 *
	 * @param string $path
	 * @param array  $params
	 *
	 * @return string
	 */
	public static function get_template_html( $path, $params = array() ) {
		ob_start();
		self::include_template( $path, $params );

		return ob_get_clean();
	}

	/**
	 * Format form posted data.
	 *
	 * @param $post_data
	 *
	 * @return array
	 */
	public static function get_formatted_form_data( $post_data ) {
		if ( empty( $post_data ) ) {
			return $post_data;
		}

		foreach ( $post_data as $key => $value ) {
			unset( $post_data[ $key ] );

			$key   = str_replace( 'iconic_ww_', '', $key );
			$value = is_numeric( $value ) ? floatval( $value ) : $value;

			$post_data[ $key ] = $value;
		}

		return $post_data;
	}

	/**
	 * Get a unique hash with key and array data.
	 *
	 * @param string $key
	 * @param array  $data
	 *
	 * @return string
	 */
	public static function get_unique_hash( $key, $data = array() ) {
		if ( empty( $data ) ) {
			return $key;
		}

		$data = json_encode( $data );

		return sprintf( '%s_%s', $key, md5( $data ) );
	}

	/**
	 * Get JSON args for attribute.
	 *
	 * @param array $args
	 *
	 * @return string
	 */
	public static function get_json_args( $args = array() ) {
		return esc_attr( json_encode( $args ) );
	}

	/**
	 * Cast value.
	 *
	 * @param mixed       $value
	 * @param string|bool $type
	 *
	 * @return mixed
	 */
	public static function cast( $value, $type ) {
		if ( ! $type ) {
			return $value;
		}

		switch ( $type ) {
			case 'int':
				$value = absint( $value );
				break;
			case 'float':
				$value = floatval( $value );
				break;
		}

		return $value;
	}

	/**
	 * Get page options.
	 *
	 * @return array
	 */
	public static function get_page_options() {
		$return = array(
			__( 'Select a page...', 'iconic-ww' ),
		);
		$pages  = get_pages( array(
			'post_status' => 'publish,private,draft',
		) );

		if ( empty( $pages ) ) {
			return $return;
		}

		foreach ( $pages as $page ) {
			$path                = get_page_uri( $page->ID );
			$return[ $page->ID ] = sprintf( '%s (/%s)', $page->post_title, $path );
		}

		return $return;
	}

	/**
	 * Output SVG.
	 *
	 * @param string      $name
	 * @param bool|string $size
	 * @param bool        $echo
	 *
	 * @return string|bool
	 */
	public static function output_svg( $name, $size = false, $echo = true ) {
		if ( empty( $name ) ) {
			return false;
		}

		$file_path = apply_filters( 'iconic_ww_svg_path', sprintf( '%sassets/svg/%s.svg', ICONIC_WW_PATH, $name ), $name );

		if ( ! file_exists( $file_path ) ) {
			return false;
		}

		$class = array(
			'iconic-ww-icon',
			'iconic-ww-icon--' . $name,
		);

		if ( $size ) {
			$class[] = 'iconic-ww-icon--' . $size;
		}

		ob_start();
		?>
		<span class="<?php echo esc_attr( implode( ' ', $class ) ); ?>">
			<?php include $file_path; ?>
		</span>
		<?php
		$svg = ob_get_clean();

		if ( ! $echo ) {
			return $svg;
		}

		echo $svg;
	}

	/**
	 * Get allowed product statues.
	 *
	 * @return array
	 */
	public static function get_allowed_product_statuses() {
		return apply_filters( 'iconic_ww_allowed_product_statuses', array( 'publish' ) );
	}

	/**
	 * Get permalink.
	 *
	 * @param WC_Product  $product
	 * @param array       $attributes
	 * @param null|string $url
	 *
	 * @return string
	 */
	public static function get_permalink( $product, $attributes = array(), $url = null ) {
		$url = ! is_null( $url ) ? $url : $product->get_permalink();

		if ( empty( $attributes ) ) {
			return $url;
		}

		$attributes = array_filter( $attributes );

		if ( empty( $attributes ) ) {
			return $url;
		}

		// Filter and encode keys and values so this is not broken by add_query_arg.
		$attributes = array_map( 'urlencode', $attributes );
		$keys       = array_map( 'urlencode', array_keys( $attributes ) );

		return add_query_arg( array_combine( $keys, $attributes ), $url );
	}
}