<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}

/**
 * Iconic_WW_User.
 *
 * @class    Iconic_WW_User
 * @version  1.0.0
 * @category Class
 * @author   Iconic
 */
class Iconic_WW_User {
	/**
	 * @var null|WP_User
	 */
	public $user = null;

	/**
	 * @var array
	 */
	public $data = array();

	/**
	 * Meta keys.
	 *
	 * @var array
	 */
	private static $meta_keys = array(
		'default_list_id' => '_iconic_ww_default_list_id',
	);

	/**
	 * Iconic_WW_Wishlist constructor.
	 *
	 * @param null|int|WP_User $the_user
	 */
	public function __construct( $the_user = null ) {
		$this->setup_props( $the_user );
	}

	/**
	 * Setup class props.
	 *
	 * @param null|int|WP_User $the_user
	 */
	public function setup_props( $the_user = null ) {
		if ( is_null( $the_user ) ) {
			$the_user = wp_get_current_user();
		} elseif ( is_int( $the_user ) ) {
			$the_user = get_user_by( 'id', $the_user );
		}

		$this->user = $the_user;
	}

	// Getters

	/**
	 * Get user data.
	 *
	 * @param string      $method
	 * @param array       $params
	 * @param bool|string $cast Cast the value
	 *
	 * @return mixed
	 */
	public function get( $method, $params = array(), $cast = false ) {
		$method_key = $cast ? sprintf( '%s_%s', $method, $cast ) : $method;
		$method_key = Iconic_WW_Helpers::get_unique_hash( $method_key, $params );

		if ( isset( $this->data[ $method_key ] ) ) {
			return apply_filters( 'iconic_ww_user_get_' . $method, $this->data[ $method_key ], $method, $params, $cast, $this );
		}

		if ( isset( $this->user->$method ) ) {
			$this->data[ $method_key ] = Iconic_WW_Helpers::cast( $this->user->$method, $cast );

			return apply_filters( 'iconic_ww_user_get_' . $method, $this->data[ $method_key ], $method, $params, $cast, $this );
		}

		$method_name = sprintf( 'get_%s', $method );

		if ( ! method_exists( __CLASS__, $method_name ) ) {
			return apply_filters( 'iconic_ww_user_get_no_method', false );
		}

		$this->data[ $method_key ] = call_user_func( array( __CLASS__, $method_name ), $params );

		if ( $cast ) {
			$this->data[ $method_key ] = Iconic_WW_Helpers::cast( $this->data[ $method_key ], $cast );
		}

		return apply_filters( 'iconic_ww_user_get_' . $method, $this->data[ $method_key ], $method, $params, $cast, $this );
	}

	/**
	 * Get default wishlist ID.
	 *
	 * @return int
	 */
	private function get_default_wishlist_id() {
		$id = (int) get_user_meta( $this->user->ID, self::$meta_keys['default_list_id'], true );

		if ( $id > 0 ) {
			// If post exists.
			$id = get_post_status( $id ) ? $id : 0;
		}

		if ( empty( $id ) ) {
			$wishlists = self::get_wishlists();

			if ( ! empty( $wishlists ) ) {
				$id = $wishlists[0]->get( 'ID' );
				$this->set( 'default_wishlist_id', array( 'id' => $id ) );
			}
		}

		return $id;
	}

	/**
	 * Get default wishlist.
	 *
	 * @return Iconic_WW_Wishlist
	 */
	private function get_default_wishlist() {
		$id = $this->get( 'default_wishlist_id' );

		return new Iconic_WW_Wishlist( $id );
	}

	/**
	 * Get wishlists.
	 *
	 * @return array
	 */
	private function get_wishlists( $params = array() ) {
		return Iconic_WW_Wishlists::get_wishlists( array(
			'author' => $this->user->ID,
			'posts_per_page' => isset( $params['limit'] ) ? $params['limit'] : 10,
		) );
	}

	// Setters

	/**
	 * Set user data.
	 *
	 * @param string $method
	 * @param array  $params
	 *
	 * @return mixed
	 */
	public function set( $method, $params = array() ) {
		$method_name = sprintf( 'set_%s', $method );

		if ( ! method_exists( __CLASS__, $method_name ) ) {
			return false;
		}

		do_action( 'iconic_ww_user_before_set_' . $method, $method, $params, $this );

		$result = call_user_func_array( array( __CLASS__, $method_name ), $params );

		do_action( 'iconic_ww_user_after_set_' . $method, $method, $params, $this, $result );

		return $result;
	}

	/**
	 * Set default wishlist ID.
	 *
	 * @param int $id
	 *
	 * @return int|bool
	 */
	private function set_default_wishlist_id( $id ) {
		return update_user_meta( $this->get( 'ID' ), self::$meta_keys['default_list_id'], $id );
	}

	// Conditionals

	/**
	 * Is this user the author of the wishlist?
	 *
	 * @param $wishlist
	 *
	 * @return bool
	 */
	public function is_author( $wishlist ) {
		if ( is_numeric( $wishlist ) ) {
			$wishlist = new Iconic_WW_Wishlist( $wishlist );
		}

		if ( ! $wishlist ) {
			return false;
		}

		$user_id = $this->get( 'ID' );

		if ( $user_id === 0 ) {
			return false;
		}

		return $user_id === $wishlist->get( 'post_author', array(), 'int' );
	}

	/**
	 * Has this user got any wishlists?
	 *
	 * @return bool
	 */
	public function has_wishlists() {
		$wishlists = Iconic_WW_Wishlists::get_wishlists();

		return ! empty( $wishlists );
	}
}