<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}

/**
 * Iconic_WW_Wishlist.
 *
 * @class    Iconic_WW_Wishlist
 * @version  1.0.0
 * @category Class
 * @author   Iconic
 */
class Iconic_WW_Wishlist {
	/**
	 * @var null|WP_Post
	 */
	public $post = null;

	/**
	 * @var array
	 */
	public $data = array();

	/**
	 * Meta keys.
	 *
	 * @var array
	 */
	private static $meta_keys = array(
		'visibility' => '_iconic_ww_visibility',
	);

	/**
	 * Iconic_WW_Wishlist constructor.
	 *
	 * @param int|WP_Post|null $the_post
	 */
	public function __construct( $the_post = null ) {
		$this->setup_props( $the_post );
	}

	/**
	 * Setup class props.
	 *
	 * @param int|WP_Post $the_post
	 */
	public function setup_props( $the_post ) {
		if ( is_null( $the_post ) ) {
			$view     = get_query_var( 'view', null );
			$the_post = ! empty( $view ) ? get_page_by_path( $view, OBJECT, Iconic_WW_Wishlists::$post_type ) : $the_post;
		}

		if ( is_numeric( $the_post ) ) {
			$the_post = get_post( $the_post );
		}

		$this->post = $the_post;
	}

	// Getters

	/**
	 * Get wishlist data.
	 *
	 * @param string      $method
	 * @param array       $params
	 * @param bool|string $cast Cast the value
	 *
	 * @return mixed
	 */
	public function get( $method, $params = array(), $cast = false ) {
		$method_key = $cast ? sprintf( '%s_%s', $method, $cast ) : $method;
		$method_key = Iconic_WW_Helpers::get_unique_hash( $method_key, $params );

		if ( isset( $this->data[ $method_key ] ) ) {
			return apply_filters( 'iconic_ww_wishlist_get_' . $method, $this->data[ $method_key ], $method, $params, $cast, $this );
		}

		if ( isset( $this->post->$method ) ) {
			$this->data[ $method_key ] = Iconic_WW_Helpers::cast( $this->post->$method, $cast );

			return apply_filters( 'iconic_ww_wishlist_get_' . $method, $this->data[ $method_key ], $method, $params, $cast, $this );
		}

		$method_name = sprintf( 'get_%s', $method );

		if ( ! method_exists( __CLASS__, $method_name ) ) {
			return apply_filters( 'iconic_ww_wishlist_get_no_method', false );
		}

		$this->data[ $method_key ] = call_user_func( array( $this, $method_name ), $params );

		if ( $cast ) {
			$this->data[ $method_key ] = Iconic_WW_Helpers::cast( $this->data[ $method_key ], $cast );
		}

		return apply_filters( 'iconic_ww_wishlist_get_' . $method, $this->data[ $method_key ], $method, $params, $cast, $this );
	}

	/**
	 * Get basic wishlist info.
	 *
	 * Use `get()` method.
	 *
	 * @param array $params
	 *
	 * @return array
	 */
	private function get_info( $params = array() ) {
		return array(
			'id'                    => $this->get( 'ID' ),
			'name'                  => $this->get( 'post_title' ),
			'privacy'               => $this->get( 'privacy' ),
			'url'                   => $this->get( 'url' ),
			'items_count'           => $this->get( 'items_count' ),
			'items_count_formatted' => $this->get( 'items_count_formatted' ),
			'created_date'          => $this->get( 'created_date' ),
			'user'                  => $this->get( 'user' ),
		);
	}

	/**
	 * Get wishlist URL.
	 *
	 * Use `get()` method.
	 *
	 * @param array $params
	 *
	 * @return false|string
	 */
	private function get_url( $params = array() ) {
		return sprintf( '%s%s/', Iconic_WW_Endpoints::get_endpoint_url( 'view' ), $this->get( 'post_name' ) );
	}

	/**
	 * Get items count.
	 *
	 * Use `get()` method.
	 *
	 * @param array $params
	 *
	 * @return int
	 */
	private function get_items_count( $params = array() ) {
		return Iconic_WW_Database::count_rows( 'items', array(
			'wishlist_id' => $this->get( 'ID' ),
		) );
	}

	/**
	 * Get items count formatted.
	 *
	 * Use `get()` method.
	 *
	 * @param array $params
	 *
	 * @return string
	 */
	private function get_items_count_formatted( $params = array() ) {
		return sprintf( '%d %s', $this->get( 'items_count' ), _n( 'Item', 'Items', $this->get( 'items_count' ), 'iconic-ww' ) );
	}

	/**
	 * Get items.
	 *
	 * Use `get()` method.
	 *
	 * @param array $params
	 *
	 * @return array|bool
	 */
	private function get_items( $params = array() ) {
		$rows = Iconic_WW_Database::get_rows( 'items', array(
			'wishlist_id' => $this->get( 'ID' ),
		) );

		if ( empty( $rows ) ) {
			return false;
		}

		$items = array();

		$date_format = get_option( 'date_format', 'j M Y' );

		foreach ( $rows as $row ) {
			$product_id = ! empty( $row->variation_id ) ? absint( $row->variation_id ) : absint( $row->item_id );
			$product    = wc_get_product( $product_id );

			if ( ! $product || $product->get_status() !== 'publish' ) {
				continue;
			}

			$attributes = array_filter( (array) maybe_unserialize( $row->attributes ) );

			if ( $product->is_type( 'variation' ) ) {
				$product->set_attributes( $attributes );
			}

			$items[] = array(
				'row'        => $row,
				'product'    => $product,
				'added'      => date( $date_format, strtotime( $row->date_added ) ),
				'attributes' => $attributes,
				'url'        => Iconic_WW_Helpers::get_permalink( $product, $attributes ),
			);
		}

		return $items;
	}

	/**
	 * Get created date.
	 *
	 * Use `get()` method.
	 *
	 * @param array $params
	 *
	 * @return false|string
	 */
	private function get_created_date( $params = array() ) {
		$format = isset( $params['format'] ) ? $params['format'] : wc_date_format();

		return get_the_date( $format, $this->get( 'ID' ) );
	}

	/**
	 * Get privacy status.
	 *
	 * Use `get()` method.
	 *
	 * @param array $params
	 *
	 * @return string
	 */
	private function get_privacy( $params = array() ) {
		$statuses    = Iconic_WW_Wishlists::get_statuses();
		$post_status = $this->get( 'post_status' );

		if ( ! isset( $statuses[ $post_status ] ) ) {
			return false;
		}

		return sprintf( '<span class="iconic-ww-badge iconic-ww-badge--%s">%s</span>', $post_status, $statuses[ $post_status ]['label'] );
	}

	/**
	 * Get user display name.
	 *
	 * Use `get()` method.
	 *
	 * @param array $params
	 *
	 * @return string
	 */
	private function get_user( $params = array() ) {
		$name        = $this->get( 'meta', array( 'key' => 'user_name' ) );
		$email       = $this->get( 'meta', array( 'key' => 'email' ) );
		$avatar_size = apply_filters( 'iconic_ww_avatar_size', 32 );
		$avatar      = get_avatar( $email, $avatar_size, '', '', array(
			'class' => 'iconic-ww-wishlist-user__img',
		) );
		$format      = '<span class="iconic-ww-wishlist-user">%s <span class="iconic-ww-wishlist-user__name">%s</span></span>';

		if ( empty( $name ) ) {
			return sprintf( $format, $avatar, __( 'Anonymous', 'iconic-ww' ) );
		}

		return sprintf( $format, $avatar, $name );
	}

	/**
	 * Get meta.
	 *
	 * Use `get()` method.
	 *
	 * @param array $params
	 *
	 * @return bool|mixed
	 */
	private function get_meta( $params = array() ) {
		if ( empty( $params['key'] ) ) {
			return false;
		}

		$key = sprintf( 'iconic_ww_%s', $params['key'] );

		return get_post_meta( $this->get( 'ID' ), $key, true );
	}

	/**
	 * Get share links.
	 *
	 * @return array
	 */
	private function get_share_links( $params = array() ) {
		$links = array(
			'twitter'    => $this->get( 'share_link', array(
				'type' => 'twitter',
			) ),
			'facebook'   => $this->get( 'share_link', array(
				'type' => 'facebook',
			) ),
			'googleplus' => $this->get( 'share_link', array(
				'type' => 'googleplus',
			) ),
			'email'      => $this->get( 'share_link', array(
				'type' => 'email',
			) ),
		);

		return array_filter( $links );
	}

	/**
	 * Get share link.
	 *
	 * @param array $params
	 *
	 * @return bool|string
	 */
	private function get_share_link( $params = array() ) {
		$link = false;

		if ( ! isset( $params['type'] ) ) {
			return $link;
		}

		switch ( $params['type'] ) {
			case 'twitter':
				$tweet = sprintf( __( 'Check out my %s on %s %s', 'iconic-ww' ), Iconic_WW_Wishlists::get_wishlists_label( 'singular' ), get_bloginfo( 'name' ), $this->get( 'url' ) );
				$link  = array(
					'link'  => sprintf( 'https://twitter.com/home?status=%s', rawurlencode( $tweet ) ),
					'title' => __( 'Share on Twitter', 'iconic-ww' ),
					'icon'  => Iconic_WW_Helpers::output_svg( 'twitter-brands', false, false ),
				);
				break;
			case 'facebook':
				$link = array(
					'link'  => sprintf( 'https://www.facebook.com/sharer/sharer.php?u=%s', rawurlencode( $this->get( 'url' ) ) ),
					'title' => __( 'Share on Facebook', 'iconic-ww' ),
					'icon'  => Iconic_WW_Helpers::output_svg( 'facebook-f-brands', false, false ),
				);
				break;
			case 'googleplus':
				$link = array(
					'link'  => sprintf( 'https://plus.google.com/share?url=%s', rawurlencode( $this->get( 'url' ) ) ),
					'title' => __( 'Share on Google+', 'iconic-ww' ),
					'icon'  => Iconic_WW_Helpers::output_svg( 'google-plus-g-brands', false, false ),
				);
				break;
			case 'email':
				$subject = sprintf( 'Have you seen my %s on %s?', Iconic_WW_Wishlists::get_wishlists_label( 'singular' ), get_bloginfo( 'name' ) );
				$body    = sprintf( __( 'Hey, Check out my %s on %s: %s', 'iconic-ww' ), Iconic_WW_Wishlists::get_wishlists_label( 'singular' ), get_bloginfo( 'name' ), $this->get( 'url' ) );
				$link    = array(
					'link'  => sprintf( 'mailto:?&subject=%s&body=%s', rawurlencode( $subject ), rawurlencode( $body ) ),
					'title' => __( 'Share via Email', 'iconic-ww' ),
					'icon'  => Iconic_WW_Helpers::output_svg( 'envelope-solid', false, false ),
				);
				break;
		}

		return $link;
	}

	// Setters

	/**
	 * Set wishlist data.
	 *
	 * @param string $method
	 * @param array  $params
	 *
	 * @return mixed
	 */
	public function set( $method, $params = array() ) {
		$method_name = sprintf( 'set_%s', $method );

		if ( ! method_exists( __CLASS__, $method_name ) ) {
			return false;
		}

		do_action( 'iconic_ww_wishlist_before_set_' . $method, $method, $params, $this );

		$result = call_user_func_array( array( __CLASS__, $method_name ), $params );

		do_action( 'iconic_ww_wishlist_after_set_' . $method, $method, $params, $this, $result );

		return $result;
	}

	/**
	 * Set visibility.
	 *
	 * @param string $visibility public|shared|private
	 *
	 * @return bool|int
	 */
	private function set_visibility( $visibility = 'public' ) {
		return update_post_meta( $this->get( 'ID' ), self::$meta_keys['visibility'], $visibility );
	}

	/**
	 * Set meta.
	 *
	 * @param string $key
	 * @param mixed  $value
	 *
	 * @return bool|int
	 */
	private function set_meta( $key, $value = '' ) {
		if ( empty( $key ) ) {
			return false;
		}

		$key = sprintf( 'iconic_ww_%s', $key );

		return update_post_meta( $this->get( 'ID' ), $key, $value );
	}

	/**
	 * Add item to wishlist.
	 *
	 * @param int $product_id
	 *
	 * @return bool
	 */
	public function add_item( $product_id, $variation_id = 0, $attributes = null ) {
		$data   = array(
			'wishlist_id'  => $this->get( 'ID' ),
			'item_id'      => $product_id,
			'user_id'      => $this->get( 'post_author' ),
			'variation_id' => $variation_id,
			'attributes'   => maybe_serialize( $attributes ),
		);
		$format = array( '%d', '%d', '%d', '%d', '%s' );

		return Iconic_WW_Database::insert( 'items', $data, $format, true );
	}

	/**
	 * Remove item from wishlist.
	 *
	 * @param int $product_id
	 *
	 * @return bool
	 */
	public function remove_item( $product_id ) {
		$data   = array(
			'wishlist_id' => $this->get( 'ID' ),
			'item_id'     => $product_id,
			'user_id'     => $this->get( 'post_author' ),
		);
		$format = array( '%d', '%d', '%d', );

		return Iconic_WW_Database::delete( 'items', $data, $format );
	}

	// Conditionals

	/**
	 * Check if wishlist is private|public|shared.
	 *
	 * @param $status_to_check
	 *
	 * @return bool
	 */
	public function is( $status_to_check ) {
		$statuses = Iconic_WW_Wishlists::get_statuses();

		if ( ! isset( $statuses[ $status_to_check ] ) ) {
			return false;
		}

		return $this->get( 'privacy' ) === $statuses[ $status_to_check ]['label'];
	}
}